@@ -480,7 +480,7 @@ def test_parquet(s3, engine, s3so, metadata_file):
     assert "part.0.parquet" in files
 
     df2 = dd.read_parquet(
-        url, index="foo", gather_statistics=True, engine=engine, storage_options=s3so
+        url, index="foo", calculate_divisions=True, engine=engine, storage_options=s3so
     )
     assert len(df2.divisions) > 1
 
@@ -573,6 +573,7 @@ def test_parquet_append(s3, engine, s3so):
         engine=engine,
         storage_options=s3so,
         write_index=False,
+        write_metadata_file=True,
     )
     df.to_parquet(
         url,
@@ -602,9 +603,10 @@ def test_parquet_append(s3, engine, s3so):
     )
 
 
-def test_parquet_wstoragepars(s3, s3so):
+@pytest.mark.parametrize("engine", ["pyarrow", "fastparquet"])
+def test_parquet_wstoragepars(s3, s3so, engine):
+    pytest.importorskip(engine)
     dd = pytest.importorskip("dask.dataframe")
-    pytest.importorskip("fastparquet")
     pd = pytest.importorskip("pandas")
     np = pytest.importorskip("numpy")
 
@@ -612,15 +614,29 @@ def test_parquet_wstoragepars(s3, s3so):
 
     data = pd.DataFrame({"i32": np.array([0, 5, 2, 5])})
     df = dd.from_pandas(data, chunksize=500)
-    df.to_parquet(url, write_index=False, storage_options=s3so)
+    df.to_parquet(
+        url,
+        engine=engine,
+        write_index=False,
+        storage_options=s3so,
+        write_metadata_file=True,
+    )
 
-    dd.read_parquet(url, storage_options=dict(**s3so, **{"default_fill_cache": False}))
+    dd.read_parquet(
+        url,
+        engine=engine,
+        storage_options=dict(**s3so, **{"default_fill_cache": False}),
+    )
     assert s3.current().default_fill_cache is False
-    dd.read_parquet(url, storage_options=dict(**s3so, **{"default_fill_cache": True}))
+    dd.read_parquet(
+        url, engine=engine, storage_options=dict(**s3so, **{"default_fill_cache": True})
+    )
     assert s3.current().default_fill_cache is True
 
     dd.read_parquet(
-        url, storage_options=dict(**s3so, **{"default_block_size": 2**20})
+        url,
+        engine=engine,
+        storage_options=dict(**s3so, **{"default_block_size": 2**20}),
     )
     assert s3.current().default_block_size == 2**20
     with s3.current().open(url + "/_metadata") as f:
