@@ -3,15 +3,11 @@ from collections.abc import Mapping
 from io import BytesIO
 from warnings import catch_warnings, simplefilter, warn
 
-from dask.highlevelgraph import HighLevelGraph
-from dask.layers import DataFrameIOLayer
-
 try:
     import psutil
 except ImportError:
     psutil = None
 
-import fsspec.implementations.local
 import numpy as np
 import pandas as pd
 from fsspec.compression import compr
@@ -30,13 +26,14 @@ from pandas.api.types import (
 from dask.base import tokenize
 from dask.bytes import read_bytes
 from dask.core import flatten
-from dask.dataframe.core import new_dd_object
+from dask.dataframe.io.io import from_map
+from dask.dataframe.io.utils import DataFrameIOFunction
 from dask.dataframe.utils import clear_known_categories
 from dask.delayed import delayed
 from dask.utils import asciitable, parse_bytes
 
 
-class CSVFunctionWrapper:
+class CSVFunctionWrapper(DataFrameIOFunction):
     """
     CSV Function-Wrapper Class
     Reads CSV data from disk to produce a partition (given a key).
@@ -55,7 +52,7 @@ class CSVFunctionWrapper:
         kwargs,
     ):
         self.full_columns = full_columns
-        self.columns = columns
+        self._columns = columns
         self.colname = colname
         self.head = head
         self.header = header
@@ -64,6 +61,10 @@ class CSVFunctionWrapper:
         self.enforce = enforce
         self.kwargs = kwargs
 
+    @property
+    def columns(self):
+        return self.full_columns if self._columns is None else self._columns
+
     def project_columns(self, columns):
         """Return a new CSVFunctionWrapper object with
         a sub-column projection.
@@ -114,14 +115,14 @@ class CSVFunctionWrapper:
         # Deal with column projection
         columns = self.full_columns
         project_after_read = False
-        if self.columns is not None:
+        if self._columns is not None:
             if self.kwargs:
                 # To be safe, if any kwargs are defined, avoid
                 # changing `usecols` here. Instead, we can just
                 # select columns after the read
                 project_after_read = True
             else:
-                columns = self.columns
+                columns = self._columns
                 rest_kwargs["usecols"] = columns
 
         # Call `pandas_read_text`
@@ -375,13 +376,8 @@ def text_blocks_to_pandas(
     for i in range(len(blocks)):
         parts.append([blocks[i], paths[i] if paths else None, is_first[i], is_last[i]])
 
-    # Create Blockwise layer
-    label = "read-csv-"
-    name = label + tokenize(reader, urlpath, columns, enforce, head, blocksize)
-    layer = DataFrameIOLayer(
-        name,
-        columns,
-        parts,
+    # Construct the output collection with from_map
+    return from_map(
         CSVFunctionWrapper(
             columns,
             None,
@@ -393,11 +389,13 @@ def text_blocks_to_pandas(
             enforce,
             kwargs,
         ),
-        label=label,
+        parts,
+        meta=head,
+        label="read-csv",
+        token=tokenize(reader, urlpath, columns, enforce, head, blocksize),
+        enforce_metadata=False,
         produces_tasks=True,
     )
-    graph = HighLevelGraph({name: layer}, {name: set()})
-    return new_dd_object(graph, name, head, (None,) * (len(blocks) + 1))
 
 
 def block_mask(block_lists):
@@ -517,6 +515,10 @@ def read_pandas(
             2
         ]
 
+        # Check for at least one valid path
+        if len(paths) == 0:
+            raise OSError(f"{urlpath} resolved to no files")
+
         # Infer compression from first path
         compression = infer_compression(paths[0])
 
@@ -829,59 +831,74 @@ def to_csv(
     >>> paths = ['/path/to/data/alice.csv', '/path/to/data/bob.csv', ...]  # doctest: +SKIP
     >>> df.to_csv(paths) # doctest: +SKIP
 
+    You can also provide a directory name:
+
+    >>> df.to_csv('/path/to/data') # doctest: +SKIP
+
+    The files will be numbered 0, 1, 2, (and so on) suffixed with '.part':
+
+    ::
+
+        /path/to/data/0.part
+        /path/to/data/1.part
+
     Parameters
     ----------
     df : dask.DataFrame
         Data to save
-    filename : string
-        Path glob indicating the naming scheme for the output files
+    filename : string or list
+        Absolute or relative filepath(s). Prefix with a protocol like ``s3://``
+        to save to remote filesystems.
     single_file : bool, default False
         Whether to save everything into a single CSV file. Under the
         single file mode, each partition is appended at the end of the
-        specified CSV file. Note that not all filesystems support the
-        append mode and thus the single file mode, especially on cloud
-        storage systems such as S3 or GCS. A warning will be issued when
-        writing to a file that is not backed by a local filesystem.
-    encoding : string, optional
-        A string representing the encoding to use in the output file,
-        defaults to 'ascii' on Python 2 and 'utf-8' on Python 3.
-    mode : str
-        Python write mode, default 'w'
+        specified CSV file.
+    encoding : string, default 'utf-8'
+        A string representing the encoding to use in the output file.
+    mode : str, default 'w'
+        Python file mode. The default is 'w' (or 'wt'), for writing
+        a new file or overwriting an existing file in text mode. 'a'
+        (or 'at') will append to an existing file in text mode or
+        create a new file if it does not already exist. See :py:func:`open`.
     name_function : callable, default None
         Function accepting an integer (partition index) and producing a
         string to replace the asterisk in the given filename globstring.
         Should preserve the lexicographic order of partitions. Not
-        supported when `single_file` is `True`.
+        supported when ``single_file`` is True.
     compression : string, optional
-        a string representing the compression to use in the output file,
+        A string representing the compression to use in the output file,
         allowed values are 'gzip', 'bz2', 'xz',
-        only used when the first argument is a filename
-    compute : bool
-        If true, immediately executes. If False, returns a set of delayed
+        only used when the first argument is a filename.
+    compute : bool, default True
+        If True, immediately executes. If False, returns a set of delayed
         objects, which can be computed at a later time.
     storage_options : dict
         Parameters passed on to the backend filesystem class.
-    header_first_partition_only : boolean, default None
-        If set to `True`, only write the header row in the first output
+    header_first_partition_only : bool, default None
+        If set to True, only write the header row in the first output
         file. By default, headers are written to all partitions under
-        the multiple file mode (`single_file` is `False`) and written
-        only once under the single file mode (`single_file` is `True`).
-        It must not be `False` under the single file mode.
+        the multiple file mode (``single_file`` is False) and written
+        only once under the single file mode (``single_file`` is True).
+        It must be True under the single file mode.
     compute_kwargs : dict, optional
         Options to be passed in to the compute method
     kwargs : dict, optional
-        Additional parameters to pass to `pd.DataFrame.to_csv()`
+        Additional parameters to pass to :meth:`pandas.DataFrame.to_csv`.
 
     Returns
     -------
-    The names of the file written if they were computed right away
-    If not, the delayed tasks associated to the writing of the files
+    The names of the file written if they were computed right away.
+    If not, the delayed tasks associated with writing the files.
 
     Raises
     ------
     ValueError
-        If `header_first_partition_only` is set to `False` or
-        `name_function` is specified when `single_file` is `True`.
+        If ``header_first_partition_only`` is set to False or
+        ``name_function`` is specified when ``single_file`` is True.
+
+    See Also
+    --------
+    fsspec.open_files
     """
     if single_file and name_function is not None:
         raise ValueError("name_function is not supported under the single file mode")
@@ -901,8 +918,6 @@ def to_csv(
     dfs = df.to_delayed()
     if single_file:
         first_file = open_file(filename, mode=mode, **file_options)
-        if not isinstance(first_file.fs, fsspec.implementations.local.LocalFileSystem):
-            warn("Appending data to a network storage system may not work.")
         value = to_csv_chunk(dfs[0], first_file, **kwargs)
         append_mode = mode.replace("w", "") + "a"
         append_file = open_file(filename, mode=append_mode, **file_options)
