@@ -90,7 +90,14 @@ class ParquetFunctionWrapper(DataFrameIOFunction):
             self.fs,
             self.engine,
             self.meta,
-            [(p["piece"], p.get("kwargs", {})) for p in part],
+            [
+                # Temporary workaround for HLG serialization bug
+                # (see: https://github.com/dask/dask/issues/8581)
+                (p.data["piece"], p.data.get("kwargs", {}))
+                if hasattr(p, "data")
+                else (p["piece"], p.get("kwargs", {}))
+                for p in part
+            ],
             self.columns,
             self.index,
             self.common_kwargs,
@@ -214,7 +221,7 @@ def read_parquet(
         of the combined filters with an OR disjunction.
 
         Predicates can also be expressed as a ``List[Tuple]``. These are evaluated
-        as an AND conjunction. To express OR in predictates, one must use the
+        as an AND conjunction. To express OR in predicates, one must use the
         (preferred for "pyarrow") ``List[List[Tuple]]`` notation.
 
         Note that the "fastparquet" engine does not currently support DNF for
@@ -250,7 +257,9 @@ def read_parquet(
         This option will be ignored if ``index`` is not specified and there is
         no physical index column specified in the custom "pandas" Parquet
         metadata. Note that ``calculate_divisions=True`` may be extremely slow
-        on some systems, and should be avoided when reading from remote storage.
+        when no global ``_metadata`` file is present, especially when reading
+        from remote storage. Set this to ``True`` only when known divisions
+        are needed for your workload (see :ref:`dataframe-design-partitions).
     ignore_metadata_file : bool, default False
         Whether to ignore the global ``_metadata`` file (when one is present).
         If ``True``, or if the global ``_metadata`` file is missing, the parquet
@@ -1234,7 +1243,7 @@ def apply_filters(parts, statistics, filters):
         filters with an OR disjunction.
 
         Predicates can also be expressed as a List[Tuple]. These are evaluated
-        as an AND conjunction. To express OR in predictates, one must use the
+        as an AND conjunction. To express OR in predicates, one must use the
         (preferred) List[List[Tuple]] notation.
 
         Note that the "fastparquet" engine does not currently support DNF for
