@@ -1,4 +1,5 @@
 import warnings
+from functools import partial
 
 import numpy as np
 import pandas as pd
@@ -442,31 +443,44 @@ def assign_index(df, ind):
     return df
 
 
-def monotonic_increasing_chunk(x):
-    data = x if is_index_like(x) else x.iloc
-    return pd.DataFrame(
-        data=[[x.is_monotonic_increasing, data[0], data[-1]]],
-        columns=["monotonic", "first", "last"],
-    )
+def _monotonic_chunk(x, prop):
+    if x.empty:
+        # if input is empty, return empty df for chunk
+        data = None
+    else:
+        data = x if is_index_like(x) else x.iloc
+        data = [[getattr(x, prop), data[0], data[-1]]]
+    return pd.DataFrame(data=data, columns=["monotonic", "first", "last"])
 
 
-def monotonic_increasing_aggregate(concatenated):
-    bounds_are_monotonic = pd.Series(
-        concatenated[["first", "last"]].to_numpy().ravel()
-    ).is_monotonic_increasing
-    return concatenated["monotonic"].all() and bounds_are_monotonic
+def _monotonic_combine(concatenated, prop):
+    if concatenated.empty:
+        data = None
+    else:
+        s = pd.Series(concatenated[["first", "last"]].to_numpy().ravel())
+        is_monotonic = concatenated["monotonic"].all() and getattr(s, prop)
+        data = [[is_monotonic, s.iloc[0], s.iloc[-1]]]
+    return pd.DataFrame(data, columns=["monotonic", "first", "last"])
 
 
-def monotonic_decreasing_chunk(x):
-    data = x if is_index_like(x) else x.iloc
-    return pd.DataFrame(
-        data=[[x.is_monotonic_decreasing, data[0], data[-1]]],
-        columns=["monotonic", "first", "last"],
-    )
+def _monotonic_aggregate(concatenated, prop):
+    s = pd.Series(concatenated[["first", "last"]].to_numpy().ravel())
+    return concatenated["monotonic"].all() and getattr(s, prop)
 
 
-def monotonic_decreasing_aggregate(concatenated):
-    bounds_are_monotonic = pd.Series(
-        concatenated[["first", "last"]].to_numpy().ravel()
-    ).is_monotonic_decreasing
-    return concatenated["monotonic"].all() and bounds_are_monotonic
+monotonic_increasing_chunk = partial(_monotonic_chunk, prop="is_monotonic_increasing")
+monotonic_decreasing_chunk = partial(_monotonic_chunk, prop="is_monotonic_decreasing")
+
+monotonic_increasing_combine = partial(
+    _monotonic_combine, prop="is_monotonic_increasing"
+)
+monotonic_decreasing_combine = partial(
+    _monotonic_combine, prop="is_monotonic_decreasing"
+)
+
+monotonic_increasing_aggregate = partial(
+    _monotonic_aggregate, prop="is_monotonic_increasing"
+)
+monotonic_decreasing_aggregate = partial(
+    _monotonic_aggregate, prop="is_monotonic_decreasing"
+)
